package w2phtml.pageSplitters;

import static w2phtml.office.XMLString.TEXT_CONTINUE_NUMBERING;
import static w2phtml.office.XMLString.TEXT_LIST_HEADER;
import static w2phtml.office.XMLString.TEXT_LIST_ITEM;
import static w2phtml.office.XMLString.TEXT_LIST;
import static w2phtml.office.XMLString.TEXT_SOFT_PAGE_BREAK;

import org.apache.commons.lang3.StringUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import pro.litvinovg.xml.Debug;
import w2phtml.office.OfficeReader;

public class ListSplitter extends BasicSplitter implements ISplitter {

	public ListSplitter(OfficeReader officeReader, SplitFactory factory) {
		super(officeReader, factory);
	}

	@Override
	public boolean Split(Node list) {
		Node parent = list.getParentNode();
		// System.out.println("----Start ListSplitter------");
		// Debug.prettyPrintXml(parent);
		Node listFirstPart = list.cloneNode(false);
		NodeList listNodes = list.getChildNodes();
		Document document = list.getOwnerDocument();
		Element softPageBreak = document.createElement(TEXT_SOFT_PAGE_BREAK);
		Node nextPageItemPart = null;
		int i = 0;
		boolean dataMoved = false;
		while (listNodes.getLength() > i) {
			Node child = listNodes.item(i);
			if (child.getNodeType() == Node.ELEMENT_NODE) {
				String nodeName = child.getNodeName();
				if (nodeName.equals(TEXT_LIST_HEADER)) {
					if (containsSPB(child)) {
						// Remove inner SPB
						removeSPB(child);
						// HACK :(
						break;
					}
					listFirstPart.appendChild(child.cloneNode(true));
					// Get next element
					i++;
				} else if (nodeName.equals(TEXT_LIST_ITEM)) {
					if (containsSPB(child)) {

						// remove SPB, move previous nodes to firstPart.
						if (factory.split(child)) {
							dataMoved = true;
							// Add first part of list item to previous list item
							listFirstPart.appendChild(child.getPreviousSibling());
							if (child.hasChildNodes()) {
								nextPageItemPart = child;
							} else {
								list.removeChild(child);
							}
						}
						// Debug.printNode(child);

						break;
					} else {
						// Not with SPB yet, move node, set dataMoved=true
						listFirstPart.appendChild(child);
						dataMoved = true;
					}
				} else {
					System.out.println("Error. SPB in List child node " + nodeName);
					System.exit(1);
				}
			}
		}
		if (dataMoved) {
			((Element) list).setAttribute(TEXT_CONTINUE_NUMBERING, "true");
			parent.insertBefore(listFirstPart, list);
			parent.insertBefore(softPageBreak, list);
			// System.out.println("LIST FIRST PART");
			// Debug.printNode(listFirstPart);
			// TODO: Create dummy list style to override list item to not teleport
			// second part but making it list item without list mark

			if (nextPageItemPart != null) {
				Node nextPageFirstNode = nextPageItemPart.getFirstChild();
					i = 0;
					NodeList freeNodes = nextPageItemPart.getChildNodes();
					while (freeNodes.getLength() > 0) {
						Node freeNode = freeNodes.item(i);
						String freeNodeName = freeNode.getNodeName();
						if (StringUtils.equals(TEXT_LIST, freeNodeName)) {
							break;
						}
						parent.insertBefore(freeNode, list);
					}
					if (!nextPageItemPart.hasChildNodes()) {
						list.removeChild(nextPageItemPart);	
					}
			}

		} else {
			// If SPB right before list items
			// parent.insertBefore(softPageBreak, list);
		}
		// System.out.println("----Finish ListSplitter------");

		// Debug.prettyPrintXml(parent);

		return dataMoved;
	}

}
